@@ -8,7 +8,7 @@ package Lemonldap::NG::Common::Apache::Session::SOAP;
 use strict;
 use SOAP::Lite;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 #parameter proxy Url of SOAP service
 #parameter proxyOptions SOAP::Lite options
@@ -147,7 +147,7 @@ sub _soapCall {
 
 ## @method hashRef get(string id)
 # @param $id Apache::Session session ID.
-# @return User datas
+# @return User data
 sub get {
     my $self = shift;
     my $id   = shift;
@@ -169,7 +169,7 @@ sub get {
 
 ## @method hashRef newSession()
 # Build a new Apache::Session session.
-# @return User datas (just the session ID)
+# @return User data (just the session ID)
 sub newSession {
     my $self = shift;
     $self->{data} = $self->_soapCall("newSession");
@@ -187,7 +187,7 @@ sub newSession {
 }
 
 ## @method boolean save()
-# Save user datas if modified.
+# Save user data if modified.
 sub save {
     my $self = shift;
     return unless ( $self->{modified} );
@@ -342,7 +342,7 @@ Lemonldap::NG Web-SSO configuration. It is used by L<Lemonldap::NG::Handler>,
 L<Lemonldap::NG::Portal> and L<Lemonldap::NG::Manager>.
 
 Lemonldap::NG::Common::Apache::Session::SOAP used with
-L<Lemonldap::NG::Portal> provides the ability to acces to
+L<Lemonldap::NG::Portal> provides the ability to access to
 Lemonldap::NG sessions via SOAP: the portal act as a proxy to access to the
 real Apache::Session module (see HTML documentation for more)
 
@@ -413,7 +413,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
