@@ -22,12 +22,12 @@ use Lemonldap::NG::Common::Crypto
   ;    #link protected cipher Object "cypher" in configuration hash
 use Config::IniFiles;
 
-#inherits Lemonldap::NG::Common::Conf::File
-#inherits Lemonldap::NG::Common::Conf::DBI
-#inherits Lemonldap::NG::Common::Conf::SOAP
-#inherits Lemonldap::NG::Common::Conf::LDAP
+#inherits Lemonldap::NG::Common::Conf::Backends::File
+#inherits Lemonldap::NG::Common::Conf::Backends::DBI
+#inherits Lemonldap::NG::Common::Conf::Backends::SOAP
+#inherits Lemonldap::NG::Common::Conf::Backends::LDAP
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 our $msg     = '';
 our $iniObj;
 
@@ -45,7 +45,7 @@ BEGIN {
 # - Nothing: default configuration file is tested,
 # - { confFile => "/path/to/storage.conf" },
 # - { Type => "File", dirName => "/path/to/conf/dir/" },
-# - { Type => "DBI", dbiChain => "DBI:mysql:database=lemonldap-ng;host=1.2.3.4",
+# - { Type => "DBI", dbiChain => "DBI:MariaDB:database=lemonldap-ng;host=1.2.3.4",
 # dbiUser => "user", dbiPassword => "password" },
 # - { Type => "SOAP", proxy => "https://auth.example.com/config" },
 # - { Type => "LDAP", ldapServer => "ldap://localhost", ldapConfBranch => "ou=conf,ou=applications,dc=example,dc=com",
@@ -97,7 +97,7 @@ sub new {
     if ( $self->{localStorage} and not defined( $self->{refLocalStorage} ) ) {
         eval "use $self->{localStorage};";
         if ($@) {
-            $msg .= "Unable to load $self->{localStorage}: $@.\n";
+            $msg .= "Error: Unable to load $self->{localStorage}: $@.\n";
         }
 
         # TODO: defer that until $> > 0 (to avoid creating local cache with
@@ -132,12 +132,18 @@ sub saveConf {
     my $tmp = $self->store($conf);
 
     unless ( $tmp > 0 ) {
-        $msg .= "Configuration $conf->{cfgNum} not stored.\n";
+        $msg .= "Error: Configuration $conf->{cfgNum} not stored.\n";
         $self->unlock();
         return ( $tmp ? $tmp : UNKNOWN_ERROR );
     }
 
     $msg .= "Configuration $conf->{cfgNum} stored.\n";
+    if ( $self->{refLocalStorage} ) {
+        $self->setDefault($conf);
+        $self->compactConf($conf);
+        $self->setLocalConf($conf);
+    }
+
     return ( $self->unlock() ? $tmp : UNKNOWN_ERROR );
 }
 
@@ -151,15 +157,15 @@ sub saveConf {
 # @return Lemonldap::NG configuration
 sub getConf {
     my ( $self, $args ) = @_;
+    my $res;
 
     # Use only cache to get conf if $args->{local} is set
     if (    $>
         and $args->{local}
         and ref( $self->{refLocalStorage} )
-        and my $res = $self->{refLocalStorage}->get('conf') )
+        and $res = $self->{refLocalStorage}->get('conf') )
     {
         $msg .= "Get configuration from cache without verification.\n";
-        return $res;
     }
 
     # Check cfgNum in conf backend
@@ -167,7 +173,7 @@ sub getConf {
     else {
         $args->{cfgNum} ||= $self->lastCfg;
         unless ( $args->{cfgNum} ) {
-            $msg .= "No configuration available in backend.\n";
+            $msg .= "Error: No configuration available in backend.\n";
         }
         my $r;
         unless ( ref( $self->{refLocalStorage} ) ) {
@@ -180,7 +186,7 @@ sub getConf {
         else {
             eval { $r = $self->{refLocalStorage}->get('conf') }
               if ( $> and not $args->{noCache} );
-            $msg = "Warn: $@" if ($@);
+            $msg .= "Warn: $@" if ($@);
             if (    ref($r)
                 and $r->{cfgNum}
                 and $args->{cfgNum}
@@ -191,8 +197,18 @@ sub getConf {
                 $args->{noCache} = 1;
             }
             else {
-                $r = $self->getDBConf($args);
-                return undef unless ( $r->{cfgNum} );
+                my $r2 = $self->getDBConf($args);
+                unless ( $r2->{cfgNum} ) {
+                    $r = $self->{refLocalStorage}->get('conf') unless ($r);
+                    $msg .=
+                      $r
+                      ? "Error: Using previous cached configuration\n"
+                      : "Error: No configuration found in local cache\n";
+                    return undef unless ($r);
+                }
+                else {
+                    $r = $r2;
+                }
 
                 $self->setDefault( $r, $args->{localPrm} );
                 $self->compactConf($r);
@@ -200,30 +216,37 @@ sub getConf {
                 # Store modified configuration in cache
                 $self->setLocalConf($r)
                   if ( $self->{refLocalStorage}
-                    and not( $args->{noCache} or $args->{raw} ) );
-
-            }
-        }
-
-        # Create cipher object
-        unless ( $args->{raw} ) {
-
-            eval {
-                $r->{cipher} = Lemonldap::NG::Common::Crypto->new( $r->{key} );
-            };
-            if ($@) {
-                $msg .= "Bad key: $@. \n";
+                    and not( $args->{noCache} == 1 or $args->{raw} ) );
             }
         }
 
         # Return configuration hash
-        return $r;
+        $res = $r;
     }
+
+    # Create cipher object
+    unless ( $args->{raw} ) {
+
+        eval {
+            $res->{cipher} = Lemonldap::NG::Common::Crypto->new( $res->{key} );
+        };
+        if ($@) {
+            $msg .= "Bad key: $@. \n";
+        }
+    }
+
+    return $res;
 }
 
 # Set default values
 sub setDefault {
     my ( $self, $conf, $localPrm ) = @_;
+    if ( defined $localPrm ) {
+        $self->{localPrm} = $localPrm;
+    }
+    else {
+        $localPrm = $self->{localPrm};
+    }
     my $defaultValues =
       Lemonldap::NG::Common::Conf::DefaultValues->defaultValues();
     if ( $localPrm and %$localPrm ) {
@@ -235,20 +258,6 @@ sub setDefault {
         $conf->{$k} //= $defaultValues->{$k};
     }
 
-    # Convert old option useXForwardedForIP into trustedProxies
-    if ( defined $conf->{useXForwardedForIP}
-        and $conf->{useXForwardedForIP} == 1 )
-    {
-        $conf->{trustedProxies} = '*';
-        delete $conf->{useXForwardedForIP};
-    }
-
-    # Force Choice backend
-    if ( $conf->{authentication} eq "Choice" ) {
-        $conf->{userDB}     = "Choice";
-        $conf->{passwordDB} = "Choice";
-    }
-
     # Some parameters expect key name (example), not variable ($example)
     if ( defined $conf->{whatToTrace} ) {
         $conf->{whatToTrace} =~ s/^\$//;
@@ -316,7 +325,13 @@ sub getLocalConf {
     if ($loaddefault) {
         foreach ( $cfg->Parameters(DEFAULTSECTION) ) {
             $r->{$_} = $cfg->val( DEFAULTSECTION, $_ );
-            if ( $r->{$_} =~ /^[{\[].*[}\]]$/ || $r->{$_} =~ /^sub\s*{.*}$/ ) {
+            if ( $_ eq "require" ) {
+                eval { require $r->{$_} };
+                $msg .= "Error: $@" if ($@);
+            }
+            if (   $r->{$_} =~ /^[{\[].*[}\]]$/
+                || $r->{$_} =~ /^sub\s*{.*}$/ )
+            {
                 eval "\$r->{$_} = $r->{$_}";
                 if ($@) {
                     $msg .= "Warning: error in file $file: $@.\n";
@@ -335,6 +350,9 @@ sub getLocalConf {
     # Load section parameters
     foreach ( $cfg->Parameters($section) ) {
         $r->{$_} = $cfg->val( $section, $_ );
+
+        # Remove spaces before and after value (#1488)
+        $r->{$_} =~ s/^\s*(.+?)\s*/$1/;
         if ( $r->{$_} =~ /^[{\[].*[}\]]$/ || $r->{$_} =~ /^sub\s*{.*}$/ ) {
             eval "\$r->{$_} = $r->{$_}";
             if ($@) {
@@ -376,10 +394,6 @@ sub getDBConf {
     my $conf = $self->load( $args->{cfgNum} );
     $msg .= "Get configuration $conf->{cfgNum}.\n"
       if ( defined $conf->{cfgNum} );
-    $self->setLocalConf($conf)
-      if (  ref($conf)
-        and $self->{refLocalStorage}
-        and not( $args->{noCache} ) );
     return $conf;
 }
 
@@ -557,7 +571,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
