@@ -1,12 +1,13 @@
 package Lemonldap::NG::Common::PSGI::Request;
 
 use strict;
+use 5.14.0;
 use Mouse;
 use JSON;
 use Plack::Request;
 use URI::Escape;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.3';
 
 our @ISA = ('Plack::Request');
 
@@ -22,24 +23,30 @@ sub BUILD {
 
 sub new {
     my $self = Plack::Request::new(@_);
-    my $tmp  = $self->script_name;
     $self->env->{REQUEST_URI} = $self->env->{X_ORIGINAL_URI}
       if ( $self->env->{X_ORIGINAL_URI} );
     $self->env->{PATH_INFO} =~ s|//+|/|g;
-    $self->env->{PATH_INFO} =~ s|^$tmp|/|;
-    $self->{uri} = uri_unescape( $self->env->{REQUEST_URI} );
-    $self->{uri} =~ s|//+|/|g;
+
+    if ( my $tmp = $self->script_name ) {
+        $self->env->{PATH_INFO} =~ s|^$tmp|/|;
+    }
+    $self->env->{PATH_INFO} ||= '/';
+    $self->env->{REQUEST_URI} =~ s|^//+|/|g;
+    $self->{uri}         = uri_unescape( $self->env->{REQUEST_URI} );
+    $self->{data}        = {};
     $self->{error}       = 0;
     $self->{respHeaders} = [];
-    return $self;
+    return bless( $self, $_[0] );
 }
 
+sub data { $_[0]->{data} }
+
 sub uri { $_[0]->{uri} }
 
 sub userData {
     my ( $self, $v ) = @_;
-    return $_[0]->{userData} = $v if ($v);
-    return $_[0]->{userData} || { _whatToTrace => $_[0]->user, };
+    return $self->{userData} = $v if ($v);
+    return $self->{userData} || { _whatToTrace => $self->{user}, };
 }
 
 sub respHeaders {
@@ -54,6 +61,7 @@ sub languages     { $_[0]->env->{HTTP_ACCEPT_LANGUAGE} }
 sub authorization { $_[0]->env->{HTTP_AUTHORIZATION} }
 sub hostname      { $_[0]->env->{HTTP_HOST} }
 sub referer       { $_[0]->env->{REFERER} }
+sub query_string  { $_[0]->env->{QUERY_STRING} }
 
 sub error {
     my ( $self, $err ) = @_;
@@ -89,7 +97,7 @@ sub jsonBodyToObj {
         $self->error('No data');
         return undef;
     }
-    my $j = eval { from_json( $self->content ) };
+    my $j = eval { from_json( $self->content, { allow_nonref => 1 } ) };
     if ($@) {
         $self->error("$@$!");
         return undef;
@@ -174,12 +182,12 @@ be pushed in response (see L<Lemonldap::NG::Common::PSGI>).
 Be careful, it contains an array reference, not a hash one because headers
 can be multi-valued.
 
-Exemple:
+Example:
 
   # Set headers
   $req->respHeaders( "Location" => "http://x.y.z/", Etag => "XYZ", );
   # Add header
-  $req->respHeaders->{"X-Key"} = "Value"; 
+  $req->respHeaders->{"X-Key"} = "Value";
 
 =head2 set_param( $key, $value )
 
@@ -212,7 +220,7 @@ contains "application/json" or "text/json").
 
 L<http://lemonldap-ng.org/>, L<Lemonldap::NG::Common::PSGI>,
 L<Lemonldap::NG::Hander::PSGI>, L<Plack::Request>,
-L<Lemonldap::NG::Portal::Main::Constants>, 
+L<Lemonldap::NG::Portal::Main::Constants>,
 
 =head1 AUTHORS
 
@@ -225,7 +233,7 @@ L<Lemonldap::NG::Portal::Main::Constants>,
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
